@@ -2,7 +2,7 @@
 ##
 ## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
 ## Copyright (C) 2004-2005 Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <nkour@jabber.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
 ## Copyright (C) 2005 Dimitur Kirov <dkirov@gmail.com>
 ## Copyright (C) 2005 Travis Shirk <travis@pobox.com>
 ## Copyright (C) 2005 Norman Rasmussen <norman@rasmussen.co.za>
@@ -19,12 +19,14 @@
 
 import xml.sax.saxutils
 import gtk
+import gtk.glade
 import gobject
 import pango
 import os
 import sys
 
 import vcard
+import dialogs
 
 
 HAS_PYWIN32 = True
@@ -37,11 +39,12 @@ if os.name == 'nt':
 		HAS_PYWIN32 = False
 
 from common import i18n
-i18n.init()
-_ = i18n._
 from common import gajim
 from common import helpers
 
+gtk.glade.bindtextdomain(i18n.APP, i18n.DIR) 
+gtk.glade.textdomain(i18n.APP)
+
 screen_w = gtk.gdk.screen_width()
 screen_h = gtk.gdk.screen_height()
 
@@ -50,6 +53,25 @@ def get_glade(file_name, root = None):
 	file_path = os.path.join(GLADE_DIR, file_name)
 	return gtk.glade.XML(file_path, root=root, domain=i18n.APP)
 
+def get_completion_liststore(entry):
+	''' create a completion model for entry widget
+	completion list consists of (Pixbuf, Text) rows'''
+	completion = gtk.EntryCompletion()
+	liststore = gtk.ListStore(gtk.gdk.Pixbuf, str)
+	
+	render_pixbuf = gtk.CellRendererPixbuf()
+	completion.pack_start(render_pixbuf, expand = False)
+	completion.add_attribute(render_pixbuf, 'pixbuf', 0)
+	
+	render_text = gtk.CellRendererText()
+	completion.pack_start(render_text, expand = True)
+	completion.add_attribute(render_text, 'text', 1)
+	completion.set_property('text_column', 1)
+	completion.set_model(liststore)
+	entry.set_completion(completion)
+	return liststore
+	
+	
 def popup_emoticons_under_button(menu, button, parent_win):
 	''' pops emoticons menu under button, which is in parent_win'''
 	window_x1, window_y1 = parent_win.get_origin()
@@ -104,8 +126,8 @@ def get_default_font():
 		# in try because daemon may not be there
 		client = gconf.client_get_default()
 
-		return helpers.ensure_unicode_string(
-			client.get_string('/desktop/gnome/interface/font_name'))
+		return client.get_string('/desktop/gnome/interface/font_name'
+			).decode('utf-8')
 	except:
 		pass
 
@@ -125,8 +147,7 @@ def get_default_font():
 			for line in file(xfce_config_file):
 				if line.find('name="Gtk/FontName"') != -1:
 					start = line.find('value="') + 7
-					return helpers.ensure_unicode_string(
-						line[start:line.find('"', start)])
+					return line[start:line.find('"', start)].decode('utf-8')
 		except:
 			#we talk about file
 			print >> sys.stderr, _('Error: cannot open %s for reading') % xfce_config_file
@@ -141,37 +162,13 @@ def get_default_font():
 					font_name = values[0]
 					font_size = values[1]
 					font_string = '%s %s' % (font_name, font_size) # Verdana 9
-					return helpers.ensure_unicode_string(font_string)
+					return font_string.decode('utf-8')
 		except:
 			#we talk about file
 			print >> sys.stderr, _('Error: cannot open %s for reading') % kde_config_file
 	
 	return None
 	
-def reduce_chars_newlines(text, max_chars = 0, max_lines = 0):
-	'''Cut the chars after 'max_chars' on each line
-	and show only the first 'max_lines'.
-	If any of the params is not present (None or 0) the action
-	on it is not performed'''
-
-	def _cut_if_long(str):
-		if len(str) > max_chars:
-			str = str[:max_chars - 3] + '...'
-		return str
-	
-	if max_lines == 0:
-		lines = text.split('\n')
-	else:
-		lines = text.split('\n', max_lines)[:max_lines]
-	if max_chars > 0:
-		if lines:
-			lines = map(lambda e: _cut_if_long(e), lines)
-	if lines:
-		reduced_text = reduce(lambda e, e1: e + '\n' + e1, lines)
-	else:
-		reduced_text = ''
-	return reduced_text
-
 def escape_for_pango_markup(string):
 	# escapes < > & ' "
 	# for pango markup not to break
@@ -186,7 +183,30 @@ def escape_for_pango_markup(string):
 	return escaped_str
 
 def autodetect_browser_mailer():
-	# recognize the environment for appropriate browser/mailer
+	# recognize the environment and set appropriate browser/mailer
+	if user_runs_gnome():
+		gajim.config.set('openwith', 'gnome-open')
+	elif user_runs_kde():
+		gajim.config.set('openwith', 'kfmclient exec')
+	elif user_runs_xfce():
+		gajim.config.set('openwith', 'exo-open')
+	else:
+		gajim.config.set('openwith', 'custom')
+
+def user_runs_gnome():
+	return 'gnome-session' in get_running_processes()
+
+def user_runs_kde():
+	return 'startkde' in get_running_processes()
+
+def user_runs_xfce():
+	procs = get_running_processes()
+	if 'startxfce4' in procs or 'xfce4-session' in procs:
+		return True
+	return False
+
+def get_running_processes():
+	'''returns running processes or None (if not /proc exists)'''
 	if os.path.isdir('/proc'):
 		# under Linux: checking if 'gnome-session' or
 		# 'startkde' programs were run before gajim, by
@@ -216,12 +236,9 @@ def autodetect_browser_mailer():
 
 		# list of processes
 		processes = [os.path.basename(os.readlink('/proc/' + f +'/exe')) for f in files]
-		if 'gnome-session' in processes:
-			gajim.config.set('openwith', 'gnome-open')
-		elif 'startkde' in processes:
-			gajim.config.set('openwith', 'kfmclient exec')
-		else:
-			gajim.config.set('openwith', 'custom')
+		
+		return processes
+	return []
 
 def move_window(window, x, y):
 	'''moves the window but also checks if out of screen'''
@@ -369,7 +386,7 @@ def possibly_move_window_in_current_desktop(window):
 	current virtual desktop
 	window is GTK window'''
 	if os.name == 'nt':
-		return
+		return False
 
 	root_window = gtk.gdk.screen_get_default().get_root_window()
 	# current user's vd
@@ -384,7 +401,9 @@ def possibly_move_window_in_current_desktop(window):
 		if current_virtual_desktop_no != window_virtual_desktop:
 			# we are in another VD that the window was
 			# so show it in current VD
-			window.show()
+			window.present()
+			return True
+	return False
 
 def file_is_locked(path_to_file):
 	'''returns True if file is locked (WINDOWS ONLY)'''
@@ -436,7 +455,7 @@ def _get_fade_color(treeview, selected, focused):
 
 def get_scaled_pixbuf(pixbuf, kind):
 	'''returns scaled pixbuf, keeping ratio etc or None
-	kind is either "chat" or "roster" or "notification" or "tooltip"'''
+	kind is either "chat", "roster", "notification", "tooltip", "vcard"'''
 	
 	# resize to a width / height for the avatar not to have distortion
 	# (keep aspect ratio)
@@ -506,7 +525,8 @@ def make_color_string(color):
 	'''create #aabbcc color string from gtk color'''
 	col = '#'
 	for i in (color.red, color.green, color.blue):
-		h = hex(i)[2:4]
+		# GTK sometime return a value > 256
+		h = hex(i%256)[2:4]
 		if len(h) == 1:
 			h = '0' + h
 		col += h
@@ -639,11 +659,94 @@ def get_state_image_from_file_path_show(file_path, show):
 def get_possible_button_event(event):
 	'''mouse or keyboard caused the event?'''
 	if event.type == gtk.gdk.KEY_PRESS:
-		event_button = 0 # no event.button so pass 0
-	else: # BUTTON_PRESS event, so pass event.button
-		event_button = event.button
-
-	return event_button
+		return 0 # no event.button so pass 0
+	# BUTTON_PRESS event, so pass event.button
+	return event.button
 
 def destroy_widget(widget):
 	widget.destroy()
+
+def on_avatar_save_as_menuitem_activate(widget, jid, account,
+default_name = ''):
+	def on_ok(widget):
+		def on_ok2(widget, file_path, pixbuf):
+			pixbuf.save(file_path, 'jpeg')
+			dialog2.destroy()
+			dialog.destroy()
+
+		file_path = dialog.get_filename()
+		file_path = decode_filechooser_file_paths((file_path,))[0]
+		if os.path.exists(file_path):
+			# check if we have write permissions
+			if not os.access(file_path, os.W_OK):
+				file_name = os.path.basename(file_path)
+				dialogs.ErrorDialog(_('Cannot overwrite existing file "%s"' % 
+					file_name),
+				_('A file with this name already exists and you do not have '
+				'permission to overwrite it.'))
+				return
+			dialog2 = dialogs.FTOverwriteConfirmationDialog(
+				_('This file already exists'), _('What do you want to do?'),
+				False)
+			dialog2.set_transient_for(dialog)
+			dialog2.set_destroy_with_parent(True)
+			response = dialog2.get_response()
+			if response < 0:
+				return
+		else:
+			dirname = os.path.dirname(file_path)
+			if not os.access(dirname, os.W_OK):
+				dialogs.ErrorDialog(_('Directory "%s" is not writable') % \
+				dirname, _('You do not have permission to create files in this'
+				' directory.'))
+				return
+
+		# Get pixbuf
+		pixbuf = None
+		is_fake = False
+		if account and gajim.contacts.is_pm_from_jid(account, jid):
+			is_fake = True
+		pixbuf = get_avatar_pixbuf_from_cache(jid, is_fake)
+		ext = file_path.split('.')[-1]
+		type_ = ''
+		if not ext:
+			# Silently save as Jpeg image
+			file_path += '.jpeg'
+			type_ = 'jpeg'
+		elif ext == 'jpg':
+			type_ = 'jpeg'
+		else:
+			type_ = ext
+
+		# Save image
+		try:
+			pixbuf.save(file_path, type_)
+		except:
+			if os.path.exists(file_path):
+				os.remove(file_path)
+			new_file_path = '.'.join(file_path.split('.')[:-1]) + '.jpeg'
+			dialog2 = dialogs.ConfirmationDialog(_('Extension not supported'),
+				_('Image cannot be saved in %(type)s format. Save as %(new_filename)s?') % {'type': type_, 'new_filename': new_file_path},
+				on_response_ok = (on_ok2, new_file_path, pixbuf))
+		else:
+			dialog.destroy()
+
+	def on_cancel(widget):
+		dialog.destroy()
+
+	dialog = dialogs.FileChooserDialog(
+		title_text = _('Save Image as...'), 
+		action = gtk.FILE_CHOOSER_ACTION_SAVE, 
+		buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
+		gtk.STOCK_SAVE, gtk.RESPONSE_OK),
+		default_response = gtk.RESPONSE_OK,
+		current_folder = gajim.config.get('last_save_dir'),
+		on_response_ok = on_ok,
+		on_response_cancel = on_cancel)
+
+	dialog.set_current_name(default_name)
+	dialog.connect('delete-event', lambda widget, event:
+		on_cancel(widget))
+
+def on_bm_header_changed_state(widget, event):
+	widget.set_state(gtk.STATE_NORMAL) #do not allow selected_state
